@@ -21,27 +21,38 @@
 
 package Sbuild;
 
-use Sbuild::Conf;
 use Sbuild::Sysconfig;
 
 use strict;
+use warnings;
 use POSIX;
 use FileHandle;
+use Filesys::Df qw();
 use Time::Local;
 
 BEGIN {
     use Exporter ();
     our (@ISA, @EXPORT);
 
-    @Sbuild::ISA = qw(Exporter);
+    @ISA = qw(Exporter);
 
-    @Sbuild::EXPORT = qw(version_less version_lesseq version_eq
-		         version_compare binNMU_version parse_date
-		         isin copy dump_file help_text version_text
-		         usage_error);
+    @EXPORT = qw($debug_level $devnull version_less version_lesseq
+		 version_eq version_compare split_version
+		 binNMU_version parse_date isin copy dump_file
+		 check_packages help_text version_text usage_error
+		 send_mail debug df);
 }
 
-my $opt_correct_version_cmp;
+our $devnull;
+my $opt_correct_version_cmp = 1;
+our $debug_level = 0;
+
+BEGIN {
+    # A file representing /dev/null
+    if (!open($devnull, '+<', '/dev/null')) {
+	die "Cannot open /dev/null: $!\n";;
+    }
+}
 
 sub version_less ($$);
 sub version_lesseq ($$);
@@ -51,14 +62,16 @@ sub do_version_cmp ($$);
 sub order ($);
 sub version_cmp_single ($$);
 sub split_version ($);
-sub binNMU_version ($$);
+sub binNMU_version ($$$);
 sub parse_date ($);
 sub isin ($@);
 sub copy ($);
 sub dump_file ($);
+sub check_packages ($$);
 sub help_text ($$);
 sub version_text ($);
 sub usage_error ($$);
+sub debug (@);
 
 sub version_less ($$) {
 	my $v1 = shift;
@@ -86,6 +99,12 @@ sub version_compare ($$$) {
 	my $rel = shift;
 	my $v2 = shift;
 
+	# If we're trying to compare versions against a provided package,
+	# always return "not satisfied", forcing an install attempt.
+	if ($v1 eq "=*=PROVIDED=*=") {
+		return 0;
+	}
+
 	if ($Sbuild::opt_correct_version_cmp) {
 		system "dpkg", "--compare-versions", $v1, $rel, $v2;
 		return $? == 0;
@@ -193,11 +212,19 @@ sub split_version ($) {
 	return( $epoch, $vers, $revision );
 }
 
-sub binNMU_version ($$) {
+sub binNMU_version ($$$) {
 	my $v = shift;
 	my $binNMUver = shift;
+	my $append_to_version = shift;
 
-	return "$v+b$binNMUver";
+	my $ver = $v;
+	if (defined($append_to_version) && $append_to_version) {
+	    $ver .= $append_to_version;
+	}
+	if (defined($binNMUver) && $binNMUver) {
+	    $ver .= "+b$binNMUver";
+	}
+	return $ver;
 }
 
 my %monname = ('jan', 0, 'feb', 1, 'mar', 2, 'apr', 3, 'may', 4, 'jun', 5,
@@ -213,7 +240,7 @@ sub parse_date ($) {
     $mon =~ y/A-Z/a-z/;
     die "Invalid month name $mon" if !exists $monname{$mon};
     $mon = $monname{$mon};
-    return timelocal($sec, $min, $hour, $day, $mon, $year);
+    return timegm($sec, $min, $hour, $day, $mon, $year);
 }
 
 sub isin ($@) {
@@ -266,6 +293,100 @@ sub dump_file ($) {
     }
 }
 
+# set and list saved package list (used by sbuild-checkpackages)
+sub check_packages ($$) {
+    my $chroot = shift;
+    my $mode = shift;
+
+    my $package_checklist = $chroot->get_conf('PACKAGE_CHECKLIST');
+    my $chroot_dir = $chroot->get('Location');
+
+    my (@status, @ref, @install, @remove);
+
+    if (! open STATUS, "grep-status -F Status -s Package ' installed' '$chroot_dir/var/lib/dpkg/status' | awk '{print \$2}' |" ) {
+	print STDERR "Can't read dpkg status file in chroot: $!\n";
+	return 1;
+    }
+    while (<STATUS>) {
+	chomp;
+	push @status, $_;
+    }
+    if (! close STATUS) {
+	print STDERR "Error reading dpkg status file in chroot: $!\n";
+	return 1;
+    }
+    @status = sort @status;
+    if (!@status) {
+	print STDERR "dpkg status file is empty\n";
+	return 1;
+    }
+
+    if ($mode eq "set") {
+	if (! open WREF, "> $chroot_dir/$package_checklist") {
+	    print STDERR "Can't write reference status file $chroot_dir/$package_checklist: $!\n";
+	    return 1;
+	}
+	foreach (@status) {
+	    print WREF "$_\n";
+	}
+	if (! close WREF) {
+	    print STDERR "Error writing reference status file: $!\n";
+	    return 1;
+	}
+    } else { # "list"
+	if (! open REF, "< $chroot_dir/$package_checklist") {
+	    print STDERR "Can't read reference status file $chroot_dir/$package_checklist: $!\n";
+	    return 1;
+	}
+	while (<REF>) {
+	    chomp;
+	    push @ref, $_;
+	}
+	if (! close REF) {
+	    print STDERR "Error reading reference status file: $!\n";
+	    return 1;
+	}
+
+	@ref = sort @ref;
+	if (!@ref) {
+	    print STDERR "Reference status file is empty\n";
+	    return 1;
+	}
+
+	print "DELETE             ADD\n";
+	print "──────────────────────────────────────\n";
+	my $i = 0;
+	my $j = 0;
+
+	while ($i < scalar @status && $j < scalar @ref) {
+
+	    my $c = $status[$i] cmp $ref[$j];
+	    if ($c < 0) {
+		# In status, not reference; remove.
+		print "$status[$i]\n";
+		$i++;
+	    } elsif ($c > 0) {
+		# In reference, not status; install.
+		print "                   $ref[$j]\n";
+		$j++;
+	    } else {
+		# Identical; skip.
+		$i++; $j++;
+	    }
+	}
+
+        # Print any remaining elements
+	while ($i < scalar @status) {
+	    print "$status[$i]\n";
+	    $i++;
+	}
+	while ($j < scalar @ref) {
+	    print "                   $ref[$j]\n";
+	    $j++;
+	}
+    }
+}
+
 sub help_text ($$) {
     my $section = shift;
     my $page = shift;
@@ -307,5 +428,62 @@ sub usage_error ($$) {
     exit 1;
 }
 
+sub send_mail {
+    my $conf = shift;
+    my $to = shift;
+    my $subject = shift;
+    my $file = shift;
+    local( *MAIL, *F );
+
+    if (!open( F, "<$file" )) {
+	warn "Cannot open $file for mailing: $!\n";
+	return 0;
+    }
+    local $SIG{'PIPE'} = 'IGNORE';
+
+    if (!open( MAIL, "|" . $conf->get('MAILPROG') . " -oem $to" )) {
+	warn "Could not open pipe to " . $conf->get('MAILPROG') . ": $!\n";
+	close( F );
+	return 0;
+    }
+
+    print MAIL "From: " . $conf->get('MAILFROM') . "\n";
+    print MAIL "To: $to\n";
+    print MAIL "Subject: $subject\n";
+    print MAIL "Content-Type: text/plain; charset=UTF-8\n";
+    print MAIL "Content-Transfer-Encoding: 8bit\n";
+    print MAIL "\n";
+    while( <F> ) {
+	print MAIL "." if $_ eq ".\n";
+	print MAIL $_;
+    }
+
+    close( F );
+    if (!close( MAIL )) {
+	warn $conf->get('MAILPROG') . " failed (exit status $?)\n";
+	return 0;
+    }
+    return 1;
+}
+
+# Note: split to stderr
+sub debug (@) {
+
+    # TODO: Add debug level checking.
+    if ($debug_level) {
+	print STDERR "D: ", @_;
+    }
+}
+
+sub df {
+    my $dir = shift;
+
+    my $stat = Filesys::Df::df($dir);
+
+    return $stat->{used} if (defined($stat));
+
+# This only happens if $dir was not a valid file or directory.
+    return 0;
+}
 
 1;
