@@ -20,18 +20,18 @@
 
 # Import default modules into main
 package main;
+use Sbuild qw($devnull);
 use Sbuild::Conf;
-use Sbuild::Log qw(open_log close_log);
-use Sbuild::Chroot;
+use Sbuild::ChrootInfoSchroot;
+use Sbuild::ChrootInfoSudo;
+use Sbuild::Sysconfig;
 
 $ENV{'LC_ALL'} = "POSIX";
-$ENV{'SHELL'} = "/bin/sh";
+$ENV{'SHELL'} = $Sbuild::Sysconfig::programs{'SHELL'};
 
 # avoid intermixing of stdout and stderr
 $| = 1;
 
-$Sbuild::Conf::verbose++;
-
 package Sbuild::Utility;
 
 use strict;
@@ -39,11 +39,10 @@ use warnings;
 
 use Sbuild::Conf;
 use Sbuild::Chroot;
-use Sbuild::Sysconfig qw($arch);
 
 sub get_dist ($);
-sub setup ($);
-sub cleanup ();
+sub setup ($$);
+sub cleanup ($);
 sub shutdown ($);
 
 my $current_session;
@@ -74,16 +73,38 @@ sub get_dist ($) {
     return $dist;
 }
 
-sub setup ($) {
+sub setup ($$) {
     my $chroot = shift;
+    my $conf = shift;
 
-    $Sbuild::Conf::nolog = 1;
-    Sbuild::Log::open_log($chroot);
+
+    $conf->set('VERBOSE', 1);
+    $conf->set('NOLOG', 1);
 
     $chroot = get_dist($chroot);
 
     # TODO: Allow user to specify arch.
-    my $session = Sbuild::Chroot::new($chroot, undef, undef);
+    my $chroot_info;
+    if ($conf->get('CHROOT_MODE') eq 'schroot') {
+	$chroot_info = Sbuild::ChrootInfoSchroot->new($conf);
+    } else {
+	$chroot_info = Sbuild::ChrootInfoSudo->new($conf);
+    }
+
+    my $session;
+
+    $session = $chroot_info->create($chroot,
+				    undef, # TODO: Add --chroot option
+				    $conf->get('ARCH'));
+
+    $session->set('Log Stream', \*STDOUT);
+
+    my $chroot_defaults = $session->get('Defaults');
+    $chroot_defaults->{'DIR'} = '/';
+    $chroot_defaults->{'STREAMIN'} = $Sbuild::devnull;
+    $chroot_defaults->{'STREAMOUT'} = \*STDOUT;
+    $chroot_defaults->{'STREAMERR'} =\*STDOUT;
+
     $Sbuild::Utility::current_session = $session;
 
     if (!$session->begin_session()) {
@@ -97,16 +118,17 @@ sub setup ($) {
     return $session;
 }
 
-sub cleanup () {
+sub cleanup ($) {
+    my $conf = shift;
+
     if (defined(&main::local_cleanup)) {
 	main::local_cleanup($Sbuild::Utility::current_session);
     }
     $Sbuild::Utility::current_session->end_session();
-    Sbuild::Log::close_log();
 }
 
 sub shutdown ($) {
-    cleanup();
+    cleanup($main::conf); # FIXME: don't use global
     exit 1;
 }
 
